// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen
// SPDX-License-Identifier: BSD-3-Clause
#include "vtkCellGridShaderVertex.h"

const char *vtkCellGridShaderVertex =
"//VTK::System::Dec\n"
"// SPDX-FileCopyrightText: Copyright (c) Ken Martin, Will Schroeder, Bill Lorensen\n"
"// SPDX-License-Identifier: BSD-3-Clause\n"
"//VTK::Camera::Dec\n"
"/// View coordinate normal for this vertex.\n"
"smooth out vec3 vertexNormalVCVS;\n"
"\n"
"uniform samplerBuffer vertices;\n"
"\n"
"// The parametric coordinates of each shape-attribute integration point.\n"
"uniform samplerBuffer cell_parametrics;\n"
"\n"
"// Tuples of (cell-id, side-id) to render.\n"
"uniform highp isamplerBuffer sides;\n"
"// Look up the offset at which local cell-side connectivity starts in \\a side_local.\n"
"uniform highp isamplerBuffer side_offsets;\n"
"// The subset of DOFs in shape_conn that correspond to each side in cells of this shape.\n"
"uniform highp isamplerBuffer side_local;\n"
"// The indices of all DOFs for all cells of this shape.\n"
"uniform highp isamplerBuffer shape_conn;\n"
"// The (x,y,z) points of each DOF in the entire mesh.\n"
"uniform samplerBuffer shape_vals;\n"
"// We may need this if the color and shape have different interpolation orders\n"
"// **and** the color attribute is continuous (i.e., shares coefficients at cell boundaries).\n"
"uniform highp isamplerBuffer color_conn;\n"
"// The coefficient of each basis function at each integration point in each cell.\n"
"uniform samplerBuffer color_vals;\n"
"// For vector- or tensor-valued field attributes, which should determine the color?\n"
"// If -1 or -2, the L1-norm or L2-norm are used.\n"
"uniform int field_component;\n"
"\n"
"flat out int cellIdVS;\n"
"flat out int sideIdVS;\n"
"\n"
"{commonDefs}\n"
"{cellEval}\n"
"{cellUtil}\n"
"\n"
"/// Parametric coordinates of this vertex.\n"
"smooth out vec3 pcoordVS;\n"
"/// View coordinate normal for this vertex.\n"
"smooth out vec3 normalVCVS;\n"
"/// View coordinate position for this vertex.\n"
"//VTK::PositionVC::Dec\n"
"\n"
"/// Store coefficients for one finite element's shape and color attributes.\n"
"///\n"
"/// These are used by downstream shader(s) to interpolate values at\n"
"/// intermediate vertices (i.e., tessellation control shader) and fragments\n"
"/// for pixel-accurate renderings. Fetching these in the vertex shader\n"
"/// saves the work of fetching them for each fragment.\n"
"flat out float shapeValuesVS[{ShapeCoeffPerCell}];\n"
"flat out float colorValuesVS[{ColorCoeffPerCell}];\n"
"\n"
"void main()\n"
"{{\n"
"  // Identify the cell ID and side ID (if applicable) we are to render.\n"
"  ivec2 cellAndSide;\n"
"  if (DrawingCellsNotSides)\n"
"  {{\n"
"     cellAndSide = ivec2(gl_InstanceID, 0);\n"
"     // These two VS outputs are to be used for picking:\n"
"     cellIdVS = cellAndSide.s;\n"
"     sideIdVS = -1;\n"
"     // Note sideIdVS != cellAndSide.t; this is so that we can properly\n"
"     // compute the cell connectivity (using the zero offset in cellAndSide.t)\n"
"     // while also computing the normal (using sideIdVS).\n"
"  }}\n"
"  else\n"
"  {{\n"
"     cellAndSide = texelFetchBuffer(sides, gl_InstanceID).st;\n"
"     // These two VS outputs are to be used for picking:\n"
"     cellIdVS = cellAndSide.s;\n"
"     sideIdVS = cellAndSide.t;\n"
"  }}\n"
"\n"
"  // Fetch the offset into the (ragged) side_local table:\n"
"  int sideRaggedOffset = texelFetchBuffer(side_offsets, {ShapeIndex}).s;\n"
"\n"
"  // Fetch point coordinates and per-integration-point field values for\n"
"  // the entire cell, passing them in bulk to the fragment shader using\n"
"  // a \"flat\" keyword.\n"
"  //\n"
"  // Doing this lookup in the vertex shader saves lots of work in the\n"
"  // fragment shader (usu. invoked many times per vertex).\n"
"  //\n"
"  // NB: Currently, a cell-grid's shape attribute *must* be continuous\n"
"  // (i.e., share degrees of freedom at cell boundaries). This makes\n"
"  // fetching shapeValuesVS much simpler than fetching colorValuesVS.\n"
"  for (int ii = 0; ii < {ShapeNumBasisFun}; ++ii)\n"
"  {{\n"
"    int vertexId = texelFetchBuffer(shape_conn, cellIdVS * {ShapeNumBasisFun} + ii).s;\n"
"    for (int jj = 0; jj < {ShapeMultiplicity}; ++jj)\n"
"    {{\n"
"      shapeValuesVS[ii * {ShapeMultiplicity} + jj] = texelFetchBuffer(shape_vals, vertexId * {ShapeMultiplicity} + jj).x;\n"
"    }}\n"
"  }}\n"
"\n"
"  if ({HaveColors})\n"
"  {{\n"
"    if ({ColorContinuous})\n"
"    {{\n"
"      // Continuous (shared) field values\n"
"      for (int ii = 0; ii < {ColorNumBasisFun}; ++ii)\n"
"      {{\n"
"        int dofId = texelFetchBuffer(color_conn, cellIdVS * {ColorNumBasisFun} + ii).s;\n"
"        for (int jj = 0; jj < {ColorMultiplicity}; ++jj)\n"
"        {{\n"
"          colorValuesVS[ii * {ColorMultiplicity} + jj] = texelFetchBuffer(color_vals, dofId * {ColorMultiplicity} + jj).x;\n"
"        }}\n"
"      }}\n"
"    }}\n"
"    else\n"
"    {{\n"
"      // Discontinuous field values\n"
"      float cv;\n"
"      for (int ii = 0; ii < {ColorNumBasisFun}; ++ii)\n"
"      {{\n"
"        for (int jj = 0; jj < {ColorMultiplicity}; ++jj)\n"
"        {{\n"
"          cv = texelFetchBuffer(color_vals, (cellIdVS * {ColorNumBasisFun} + ii) * {ColorMultiplicity} + jj).x;\n"
"          colorValuesVS[ii * {ColorMultiplicity} + jj] = cv;\n"
"        }}\n"
"      }}\n"
"    }}\n"
"  }}\n"
"\n"
"  // Now compute the location of the current vertex inside the current side or cell.\n"
"  // NB: {{SideOffset}} = {SideOffset} is the offset to apply to side IDs so we can\n"
"  // look up their connectivity offset from sideRaggedOffset (the start in side_local\n"
"  // for connectivity of sides of our type).\n"
"  int sideVertexIndex =\n"
"    texelFetchBuffer(side_local,\n"
"      sideRaggedOffset + (cellAndSide.t - {SideOffset}) * NumPtsPerSide + gl_VertexID).s;\n"
"  int vertexId = texelFetchBuffer(shape_conn, cellIdVS * NumPtsPerCell + sideVertexIndex).s;\n"
"  // Parametric coordinate for this vertex.\n"
"  pcoordVS = texelFetchBuffer(cell_parametrics, sideVertexIndex).xyz;\n"
"  // position for this vertex as defined in vtk data model.\n"
"  vec4 vertexMC = vec4(\n"
"      texelFetchBuffer(shape_vals, vertexId * 3).x,\n"
"      texelFetchBuffer(shape_vals, vertexId * 3 + 1).x,\n"
"      texelFetchBuffer(shape_vals, vertexId * 3 + 2).x,\n"
"      1.0f);\n"
"  // default eye direction in model coordinates.\n"
"  vec3 eyeNormalMC = vec3(0.0f, 0.0f, 1.0f);\n"
"  vec3 vertexNormalMC = normalToSideAt(sideIdVS, shapeValuesVS, pcoordVS, -eyeNormalMC);\n"
"\n"
"  // Transform the vertex by the model-to-device coordinate matrix.\n"
"  // This matrix must be the result of the following multiplication:\n"
"  // MCDCMatrix = ModelToWorld X WorldToView X ViewToDisplay\n"
"  gl_Position = MCDCMatrix * vertexMC;\n"
"\n"
"  // Transform vertex posittion to view coordinate.\n"
"  // Some operations in fragment shader want it that way.\n"
"  vertexPositionVCVS = MCVCMatrix * vertexMC;\n"
"\n"
"  // Normal vectors are transformed in a different way than vertices.\n"
"  // Instead of pre-multiplying with MCDCMatrix, a different matrix is used.\n"
"  // This `normalMatrix` is computed on the CPU. It must be the result of the following:\n"
"  // normalMatrix = inverse(ModelToWorld X WorldToView)\n"
"  // Read more about normal matrix at http://www.songho.ca/opengl/gl_normaltransform.html\n"
"  if ((DrawingCellsNotSides && NumPtsPerCell <= 2) || (!DrawingCellsNotSides && NumPtsPerSide <= 2))\n"
"  {{\n"
"    // for lines or vertices, the normal will always face the camera.\n"
"    vertexNormalVCVS = vec3(vertexNormalMC.xy, 1.0f);\n"
"  }}\n"
"  else\n"
"  {{\n"
"    vertexNormalVCVS = normalMatrix * vertexNormalMC;\n"
"  }}\n"
"}}\n"
"";
